/*
Visualizations to look at when bored
Updated: Dec. 16, 2005.
By: Danny Tenedorio
*/

// included headers
#include "allegro.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "math.h"

// defined constants
#define PI				3.141592654
#define TRUE			-1
#define FALSE			0
#define MAX_MODES		4
#define BLACK			getcol(0, 0, 0)
#define WHITE			getcol(255, 255, 255)
#define PAUSE			clear_keybuf();	readkey()
// mode 1
#define MAX_ELIPSES		256
#define MAX_SIZE		23
// mode 2
#define MAX_TRIANGLES		600
#define MAX_TRIANGLE_SIZE	30
#define SCARE_DISTANCE		60
// mode 3
#define MAX_LINES		1600
// mode 4
#define MAX_POINTS		6
#define MAX_TRAILS		64
#define BALL_SIZE		25
#define MAX_VELOCITY		2
#define VELOCITY_INCREMENT	.1
#define VELOCITY_INCREMENT2	.1

// global structures
typedef struct  {
	int x, y;
	float rx, ry;
	float r, g, b;
	int alive;
} SCREEN_ELIPSE;

typedef struct  {
	float x, y;
	float radius;
	int r, g, b;
	int angle;
	int death_counter;
	int alive;
} SCREEN_TRIANGLE;

typedef struct  {
	int x1, y1, x2, y2;
	int r, g, b;
	int alive;
} SCREEN_LINE;

typedef struct  {
	float x, y;
	float x_velocity, y_velocity;
	int r, g, b;
	int past_x[MAX_TRAILS], past_y[MAX_TRAILS];
} SCREEN_POINT;

// function prototype definitions
void init_beginning (void);
void init_mode (int which_mode);
void load_palette (char *filename);
void load_sprites (void);
void put_sprite (int x, int y, int which);
void respond_to_input (void);
void draw_screen (void);
void copy_page (BITMAP *source, BITMAP *dest);
void copy_elipse (int one, int two);
void copy_line (int one, int two);
void cycle_elipses_up (void);
void cycle_lines_up (void);
int round(double arg);
void get_circle_point (int center_x, int center_y, double radius,
 		       int angle, int *x, int *y);
void update_variables (void);
int get_angle (int angle);
void wipe_to (int color);
long getcol (int r, int g, int b);
void draw_cursor (int yes_no);
void intro_screen (void);

// global variables
int screen_x_size = 640, screen_y_size = 480;
int game_counter = 0;
int cursor_x = 0, cursor_y = 0;
int moved_x, moved_y;
int space_down, mouse_l_down = FALSE, mouse_r_down = FALSE;
int mode = 1; // drawing mode
int type = 1; // setting within drawimg mode
int max_types;
int prev_line;
int acquire_mode = FALSE;

// global arrays
float sine[360];		// holds the trig tables
float cosine[360];
float tangent[360];

// global objects
PALETTE pal;				// stores the current palette in memory
FONT *game_font;			// font object to hold the nifty game font
FONT *title_font;			// font object to hold the nifty game font
BITMAP *buffer;				// memory buffer to which we send the data
BITMAP *second_buffer;		
DATAFILE *dat;				// datafile object that holds the datafile info

SCREEN_ELIPSE	elipse[MAX_ELIPSES];
SCREEN_TRIANGLE tri[MAX_TRIANGLES];
SCREEN_LINE	lin[MAX_LINES];
SCREEN_POINT	pt[MAX_POINTS];

int main (void)
{
	init_beginning();
	intro_screen();
	init_mode(mode);
	
	while (TRUE)
	{
		respond_to_input ();
		update_variables ();
		draw_screen ();
		copy_page (buffer, screen);
		rest(2);
		
		if ((mode == 4)) draw_cursor (FALSE);
	}
	
}

END_OF_MAIN ();



// does all the initializing
void init_beginning (void)
{
	int index;
	
	// initializations
	allegro_init();
	install_keyboard();
	install_mouse();
	install_timer();

	// init graphics mode
	set_color_depth (16);
	if (set_gfx_mode(GFX_AUTODETECT_FULLSCREEN, screen_x_size, screen_y_size, 0, 0) != 0)	{
		allegro_message("Error setting graphics mode\n%s\n", allegro_error);
		exit (1);
 	}
	
	dat = load_datafile ("font.dat");	// load the font
	game_font = (FONT *)dat[0].dat;
	title_font = (FONT *)dat[1].dat;

	// make an off-screen buffer in system RAM	
	buffer = create_bitmap(screen_x_size, screen_y_size);
	second_buffer = create_bitmap(screen_x_size, screen_y_size);
	clear_bitmap (buffer);				// clear the buffer to color 0
	clear_bitmap (screen);				// clear the screen to color 0	

	// load the trig tables
	for (index = 0; index < 360; index++)  {
		sine[index] =		sin(index * (PI / 180));
		cosine[index] =		cos(index * (PI / 180));
		tangent[index] =	tan(index * (PI / 180));
	}
	
	// set transparent text
	text_mode(-1);
	
	return;
} // init_beginning



void init_mode (int which_mode)
{
	int index, counter;
	
	switch (which_mode)  {
	case 1:  // red circles on a white background
		// make the screen white
		rectfill (buffer, 0, 0, screen_x_size - 1, screen_y_size - 1, WHITE);
		// erase the elipse array
		for (index = 1; index < MAX_ELIPSES; index++)
			elipse[index].alive = FALSE;
		elipse[1].alive = TRUE;

		break;
	case 2:	// spinning triangles
		// make the screen white
		rectfill (second_buffer, 0, 0, screen_x_size - 1, screen_y_size - 1, WHITE);
		// erase the triangle array
		for (index = 1; index < MAX_ELIPSES; index++)
			tri[index].alive = FALSE;
		// seed the random number generator
		srand (time (NULL));

		break;
	case 3:	// tear in the screen
		// make the screen white
		rectfill (buffer, 0, 0, screen_x_size - 1, screen_y_size - 1, WHITE);
		//wipe_to (getcol(0, 0, 0));

		// erase the line array
		for (index = 1; index < MAX_LINES; index++)
			lin[index].alive = FALSE;
		// init the first line
		lin[0].x1 = mouse_x;
		lin[0].y1 = mouse_y;
		lin[0].x2 = mouse_x;
		lin[0].y2 = mouse_y;
		prev_line = 0;

		break;
	case 4:	// moving points
		// make the screen white
		rectfill (buffer, 0, 0, screen_x_size - 1, screen_y_size - 1, WHITE);
		
		// seed the random number generator
		srand (time (NULL));

		// generate the points
		for (index = 0; index < MAX_POINTS; index++)  {
			pt[index].x = rand() % screen_x_size;
			pt[index].y = rand() % screen_y_size;
			pt[index].x_velocity = -(float)((rand() % 10) - 0) / 10;
			pt[index].y_velocity = -(float)((rand() % 10) - 0) / 10;
			switch (index + 1)  {
			case 1:
				pt[index].r = 255;
				pt[index].g = 0;
				pt[index].b = 0;
				break;
			case 2:
				pt[index].r = 0;
				pt[index].g = 255;
				pt[index].b = 0;
				break;
			case 3:
				pt[index].r = 0;
				pt[index].g = 0;
				pt[index].b = 255;
				break;
			case 4:
				pt[index].r = 255;
				pt[index].g = 255;
				pt[index].b = 0;
				break;
			case 5:
				pt[index].r = 255;
				pt[index].g = 0;
				pt[index].b = 255;
				break;
			case 6:
				pt[index].r = 0;
				pt[index].g = 255;
				pt[index].b = 255;
				break;
			case 7:
				pt[index].r = 0;255;
				pt[index].g = 0;128;
				pt[index].b = 0;
				break;
			}
			for (counter = 0; counter <= MAX_TRAILS; counter++)  {
				pt[index].past_x[counter] = pt[index].x;
				pt[index].past_y[counter] = pt[index].y;
			}
		}
	
	default: ;
	}

}



// responds to input
void respond_to_input (void)
{
	int index;

	poll_keyboard();
	
	if (key[KEY_RIGHT] & !key[KEY_LEFT])	 {				// if the right arrow key is pressed
	}
	else if (key[KEY_LEFT] & !key[KEY_RIGHT])  { 			// if the left arrow key is pressed
	}

	if (key[KEY_SPACE])  {							// if the space key is pressed
		if (space_down == FALSE)  {					// and it was released last loop
			if( acquire_mode == TRUE )	acquire_mode = FALSE; else acquire_mode = TRUE;
			textprintf(buffer, font, 0, 0, 15, "%d", acquire_mode);
		}
		space_down = TRUE;
	}
	else
		space_down = FALSE;
	
	if (key[KEY_P])
		readkey();
	if (key[KEY_S])
		save_bmp("scrnshot.bmp", screen, pal);				// save the bitmap to file		
	if (key[KEY_ESC]) {
		exit (0);		
	}

	// left mouse button: mode dependent
	if (mouse_b & 1)  {

		if (mouse_l_down == FALSE)  {
			switch (mode)  {
			case 4:
				for (index = 0; index < MAX_POINTS; index++)  {
					pt[index].x_velocity = (((rand() % 16) / 5) - 1) * 3;
					pt[index].y_velocity = (((rand() % 16) / 5) - 1) * 3;
				}
				break;
			}
		}
		mouse_l_down = TRUE;
	}
	else  {
		mouse_l_down = FALSE;
	}
	
	
	// right mouse button: switch modes
	if (mouse_b & 2)  {
		if (mouse_r_down == FALSE)  {
			mode++;
			if (mode > MAX_MODES) mode = 1;
			init_mode (mode);
		}
		mouse_r_down = TRUE;
	}
	else  {
		mouse_r_down = FALSE;
	}

}



// loads the sprites from the specified files
void load_sprites (void)
{
	DATAFILE *dat;

	dat = load_datafile("sprites.dat");	// load the data file
	if (!dat)  {							// if the load failed, display error message
		allegro_message("Error loading image datafile sprites.dat\n%s\n", allegro_error);
		exit (1);							// quit the program
	}

} // load_sprites



// copies one video page to another
void copy_page (BITMAP *source, BITMAP *dest)  {
	blit (source, dest, 0, 0, 0, 0, screen_x_size, screen_y_size);
}



void draw_screen (void)
{
	int index;
	int corner[6];
	int counter, counter2, col;
	
	if( acquire_mode == TRUE )	acquire_bitmap (buffer);
	
	switch (mode)  {
	case 1:
		// go through all the elipses
		for (index = MAX_ELIPSES - 1; index >= 1; index--)  {
			if (elipse[index].alive == TRUE)  {
				ellipsefill(buffer, elipse[index].x, elipse[index].y, (int)elipse[index].rx, (int)elipse[index].ry,
					getcol(elipse[index].r, elipse[index].g, elipse[index].b));
			}
		}
		
		break;

	case 2:
		copy_page (second_buffer, buffer);

		// go through all the triangles
		for (index = 1; index < MAX_TRIANGLES; index++)  {
			if ((tri[index].alive == TRUE) && ((tri[index].r == 255) && (tri[index].g == 255) && (tri[index].b > 00)))  {
// 				// get the points
				get_circle_point (tri[index].x, tri[index].y, tri[index].radius, tri[index].angle, &corner[0], &corner[1]);
				get_circle_point (tri[index].x, tri[index].y, tri[index].radius, get_angle(tri[index].angle + 120),
					&corner[2], &corner[3]);
				get_circle_point (tri[index].x, tri[index].y, tri[index].radius, get_angle(tri[index].angle + 240),
					&corner[4], &corner[5]);

				// draw the triangle
				polygon(buffer, 3, corner, getcol (tri[index].r, tri[index].g, tri[index].b));
			}
		}
				
		// go through all the triangles
		for (index = 1; index < MAX_TRIANGLES; index++)  {
			if ((tri[index].alive == TRUE) && !((tri[index].r == 255) && (tri[index].g == 255) && (tri[index].b > 00)))  {
				// get the points
				get_circle_point (tri[index].x, tri[index].y, tri[index].radius, tri[index].angle, &corner[0], &corner[1]);
				get_circle_point (tri[index].x, tri[index].y, tri[index].radius, get_angle(tri[index].angle + 120),
					&corner[2], &corner[3]);
				get_circle_point (tri[index].x, tri[index].y, tri[index].radius, get_angle(tri[index].angle + 240),
					&corner[4], &corner[5]);

				// draw the triangle
				polygon(buffer, 3, corner, getcol (tri[index].r, tri[index].g, tri[index].b));
			}
		}

		draw_cursor (TRUE);

		break;

	case 3:
		// go through all the lines
		for (index = MAX_LINES - 1; index >= 0; index--)  {
			if (lin[index].alive == TRUE) {
					// draw the line!
					line (buffer, lin[index].x1, lin[index].y1, lin[index].x2, lin[index].y2, getcol(lin[index].r,
 						lin[index].g, lin[index].b));
					//rectfill (buffer, lin[index].x1, lin[index].y1, lin[index].x2 + 30, lin[index].y2 + 30,
 					//	getcol(lin[index].r, lin[index].g, lin[index].b));

					// draw the fadey crap below it
					for (counter = 1, counter2 = 1; counter < 255 - lin[index].r; counter += 3, counter2++)  {
						col = getcol(lin[index].r + counter, 255, lin[index].b + counter);
						line (buffer, lin[index].x1, lin[index].y1 + counter2 + 0, lin[index].x2, lin[index].y2 +
 							counter2 + 0, col);	
					}

					/*// draw the fadey crap below it
					for (counter = 1, counter2 = 1; counter < lin[index].r; counter += 3, counter2++)  {
						line (buffer, lin[index].x1 + counter2, lin[index].y1, lin[index].x2 + counter2,
 							lin[index].y2, getcol(lin[index].r - counter, lin[index].g - counter, lin[index].b));
					}*/
			}

		}

		break;

	case 4:
		// go through all the points
		for (index = MAX_POINTS - 1; index >= 0; index--)  {
			for (counter = MAX_TRAILS - 1; counter > 0; counter--)  {
			
				col = getcol(pt[index].r + ( ((counter + 1) * (255 - pt[index].r)) / MAX_TRAILS),
							 pt[index].g + ( ((counter + 1) * (255 - pt[index].g)) / MAX_TRAILS),
							 pt[index].b + ( ((counter + 1) * (255 - pt[index].b)) / MAX_TRAILS));
				putpixel (buffer, pt[index].past_x[counter], pt[index].past_y[counter], col);
				circlefill (buffer, pt[index].past_x[counter], pt[index].past_y[counter], BALL_SIZE, col);
			}
		}
		
		draw_cursor (TRUE);		
		
		break;

	default: ;
	}

	if( acquire_mode == TRUE )	release_bitmap (buffer);
}



void copy_elipse (int one, int two)
{
	// copy one to two
	elipse[two].x = elipse[one].x;
	elipse[two].y = elipse[one].y;
	elipse[two].rx = elipse[one].rx;
	elipse[two].ry = elipse[one].ry;
	elipse[two].r = elipse[one].r;
	elipse[two].g = elipse[one].g;
	elipse[two].b = elipse[one].b;
	elipse[two].alive = elipse[one].alive;
}



void copy_line (int one, int two)
{
	// copy one to two
	lin[two].x1 = lin[one].x1;
	lin[two].y1 = lin[one].y1;
	lin[two].x2 = lin[one].x2;
	lin[two].y2 = lin[one].y2;
	lin[two].r = lin[one].r;
	lin[two].g = lin[one].g;
	lin[two].b = lin[one].b;
	lin[two].alive = lin[one].alive;
}



void cycle_elipses_up (void)
{
	int index;

	for (index = MAX_ELIPSES - 1; index >= 1; index--)  {
		copy_elipse (index, index + 1);
		switch (mode)  {
		case 1:
			elipse[index + 1].g = index - 1;
			elipse[index + 1].b = index - 1;
			
			if (elipse[index + 1].rx < MAX_SIZE)
				elipse[index + 1].rx += 1;
			else
				elipse[index + 1].rx += .1;

			if (elipse[index + 1].ry < MAX_SIZE)
				elipse[index + 1].ry += 1;
			else
				elipse[index + 1].ry += .1;

			
			break;
		default: ;		
		}
	}

}



void cycle_lines_up (void)
{
	int index;

	for (index = MAX_LINES - 1; index >= 1; index--)  {
		copy_line (index, index + 1);
	}

}



void update_variables (void)
{

	int index, counter;
	
	cursor_x = mouse_x;
	cursor_y = mouse_y;

	game_counter++;
	if (game_counter > INT_MAX) game_counter = 0;

	switch (mode)  {
	case 1: 

		elipse[1].x = cursor_x;
		elipse[1].y = cursor_y;
		elipse[1].rx = 8;
		elipse[1].ry = 8;
		elipse[1].r = 255;
		elipse[1].g = 0;
		elipse[1].b = 0;

		cycle_elipses_up ();
	
	case 2:

		if (game_counter % 4 == 0)  {
			// find the next open triangle spot
			for (index = 0; index < MAX_TRIANGLES; index++)  {
				if (tri[index].alive == FALSE) break;
			}
				
			// init a new triangle
			tri[index].alive = TRUE;
			tri[index].x = mouse_x;
			tri[index].y = mouse_y;
			tri[index].radius = 1;
			tri[index].r = 0;
			tri[index].g = 0;
			tri[index].b = 126 + 126;//(rand() % 126);
			tri[index].death_counter = 0;
			tri[index].angle = rand() % 360;

		}
		
		
		// all the triangles
		for (index = 0; index < MAX_TRIANGLES; index++)  {
			if (tri[index].alive == TRUE)  {
	
				if ((tri[index].x < -30) || (tri[index].x > screen_x_size + 30) || (tri[index].y < -30) || (tri[index].y >
					screen_y_size + 30))
					tri[index].alive = FALSE;
				
				if ((tri[index].r == 255) && (tri[index].g == 255))  {
					if (tri[index].b < 255)  {
						tri[index].b += 2;
						if (tri[index].b > 255) tri[index].b = 255;

					}
					else  {
						tri[index].death_counter++;
						if (tri[index].death_counter == 30)
							tri[index].alive = FALSE;
					}
				}
				
				else if (tri[index].r < 255)  {
					tri[index].r += 2;
					if (tri[index].r > 255) tri[index].r = 255;
				}
				else if (tri[index].b > 0)  {
					tri[index].b -= 2;
					if (tri[index].b < 0) tri[index].b = 0;
				}
				else if (tri[index].g < 255)  {
					tri[index].g += 2;
					if (tri[index].g > 255) tri[index].g = 255;
				}

				// expand them
				tri[index].radius += 1;
				if (tri[index].radius >= MAX_TRIANGLE_SIZE)
					tri[index].radius = MAX_TRIANGLE_SIZE;
				// rotate them
				tri[index].angle += 2;
				if (tri[index].angle >= 360)
					tri[index].angle -= 360;
				// move them
				if (!(mouse_b & 1))  {
					//D = sqrt( pow((tri[index].x - (screen_x_size / 2)), 2) + pow((tri[index].y - (screen_y_size / 2)), 2) );
					if (tri[index].x <= (screen_x_size / 2))  {
						tri[index].x -= ((screen_x_size / 2) - tri[index].x) / (screen_x_size / 4);
					}
					else  {
						tri[index].x += (tri[index].x - (screen_x_size / 2)) / (screen_x_size / 4);
					}
					if (tri[index].y <= (screen_y_size / 2))  {
						tri[index].y -= ((screen_y_size / 2) - tri[index].y) / (screen_y_size / 3);
					}
					else  {
						tri[index].y += (tri[index].y - (screen_y_size / 2)) / (screen_y_size / 3);
					}
				}

			}
		}
	
		break;
	
	case 3:
		
		// make a line from the previous one to the mouse cursor
		lin[1].x1 = lin[prev_line].x2;
		lin[1].y1 = lin[prev_line].y2;
		lin[1].x2 = cursor_x;
		lin[1].y2 = cursor_y;
		lin[1].r = 0;				// bright yellow color
		lin[1].g = 255;
		lin[1].b = 0;
		lin[1].alive = TRUE;
		prev_line = 1;

		cycle_lines_up();

		// go thru all the lines and fade them down...
		for (index = 0; index < MAX_LINES; index++)  {
			if (lin[index].alive == TRUE)  { 
				lin[index].r += 4;
				lin[index].b += 4;
				if (lin[index].r >= 255)  {
					lin[index].alive = FALSE;
				}
			}
		}
		break;

	case 4:

		for (index = 0; index < MAX_POINTS; index++)  {
			pt[index].past_x[0] = (int)pt[index].x;
			pt[index].past_y[0] = (int)pt[index].y;
			for (counter = MAX_TRAILS - 1; counter >= 1; counter--)  {
				pt[index].past_x[counter] = pt[index].past_x[counter - 1];
				pt[index].past_y[counter] = pt[index].past_y[counter - 1];
			}
			pt[index].x += pt[index].x_velocity;
			pt[index].y += pt[index].y_velocity;
			
			// change the ball's velocity based on its position relative to the mouse pointer
/*			if (mouse_b & 1)  {
				if (pt[index].x < cursor_x)			pt[index].x_velocity -= VELOCITY_INCREMENT;
				if (pt[index].x > cursor_x)			pt[index].x_velocity += VELOCITY_INCREMENT;
				if (pt[index].y < cursor_y)			pt[index].y_velocity -= VELOCITY_INCREMENT;
				if (pt[index].y > cursor_y)			pt[index].y_velocity += VELOCITY_INCREMENT;;
			}
			else  {*/
				if (pt[index].x > cursor_x)			pt[index].x_velocity -= VELOCITY_INCREMENT;
				if (pt[index].x < cursor_x)			pt[index].x_velocity += VELOCITY_INCREMENT;
				if (pt[index].y > cursor_y)			pt[index].y_velocity -= VELOCITY_INCREMENT;
				if (pt[index].y < cursor_y)			pt[index].y_velocity += VELOCITY_INCREMENT;;
//			}
			
			// change the ball's velocity based if it is outside the screen
			if (pt[index].x < 0 && pt[index].x_velocity < 0) 
				pt[index].x_velocity += VELOCITY_INCREMENT2;
			if (pt[index].x > screen_x_size	&& pt[index].x_velocity > 0)
				pt[index].x_velocity -= VELOCITY_INCREMENT2;
			if (pt[index].y < 0 && pt[index].y_velocity < 0)
				pt[index].y_velocity += VELOCITY_INCREMENT2;
			if (pt[index].y > screen_y_size && pt[index].y_velocity > 0)
				pt[index].y_velocity -= VELOCITY_INCREMENT2;
		
		}
		break;
	}

}



void get_circle_point (int center_x, int center_y, double radius, int angle, int *x, int *y)
{
	*x = center_x + round(radius * cosine[angle]);
	*y = center_y + round(radius * sine[angle]);
}


// rounds a decimal number to the nearest integer
int round(double arg)
{
	int base = floor(arg);
	if (arg - base < .5)
		arg = base;
	else
		arg = base + 1;

	return arg;
}



// returns an angle in the range 0-359
int get_angle (int angle)
{
	while (angle >= 360)
		angle -= 360;
	while (angle < 0)
		angle += 360;

	return angle;
}



void wipe_to (int color)  {

	int counter, x, y;

	for (counter = 0; counter < 40; counter++)  {
		for (y = 0; y < screen_y_size; y += 80)  {
			for (x = 0; x < screen_x_size; x += 80)  {
				line (screen, x + counter, y, x + counter, y + 39, color);
			}
		}
		for (y = 40; y < screen_y_size; y += 80)  {
			for (x = 40; x < screen_x_size; x += 80)  {
				line (screen, x + counter, y, x + counter, y + 39, color);
			}
		}
		rest(10);
	}
	for (counter = 0; counter < 40; counter++)  {
		for (y = 0; y < screen_y_size; y += 80)  {
			for (x = 79; x < screen_x_size; x += 80)  {
				line (screen, x - counter, y, x - counter, y + 39, color);
			}
		}
		for (y = 40; y < screen_y_size; y += 80)  {
			for (x = 39; x < screen_x_size; x += 80)  {
				line (screen, x - counter, y, x - counter, y + 39, color);
			}
		}
		rest(10);
	}

}



long getcol (int r, int g, int b) 
{

	return makecol16 (r, g, b);

}



void draw_cursor (int yes_no)
{
	int col;

	if (yes_no == TRUE)	 col = getcol(0, 0, 0);
	if (yes_no == FALSE) col = WHITE;

	// draw the mouse cursor
	line (buffer, cursor_x - 6, cursor_y, cursor_x - 1, cursor_y, col);
	line (buffer, cursor_x, cursor_y - 6, cursor_x, cursor_y - 1, col);
	line (buffer, cursor_x + 1, cursor_y, cursor_x + 6, cursor_y, col);
	line (buffer, cursor_x, cursor_y + 1, cursor_x, cursor_y + 6, col);

}



void intro_screen (void)
{
	int index;

	// clear the screen to white
	rectfill (buffer, 0, 0, screen_x_size - 1, screen_y_size - 1, WHITE);

	// display some messages
	textprintf_centre (buffer, game_font, screen_x_size / 2, screen_y_size / 5, BLACK, "Interactive Visuals");

	textprintf_centre (buffer, game_font, screen_x_size / 2, screen_y_size / 5 + 24, BLACK,
		"The visuals change depending on the way you move the mouse.");
	textprintf_centre (buffer, game_font, screen_x_size / 2, screen_y_size / 5 + 36, BLACK,
		"To change sets, click the right mouse button.");
	textprintf_centre (buffer, game_font, screen_x_size / 2, screen_y_size / 5 + 48, BLACK,
		"For unpredictable effects, click the left mouse button.");
	textprintf_centre (buffer, game_font, screen_x_size / 2, screen_y_size / 5 + 60, BLACK,
		"To save a screenshot, press S.");
	textprintf_centre (buffer, game_font, screen_x_size / 2, screen_y_size / 5 + 72, BLACK,
		"To pause, press P.");
	textprintf_centre (buffer, game_font, screen_x_size / 2, screen_y_size / 5 + 84, BLACK,
		"To exit, press Esc.");
	textprintf_centre (buffer, game_font, screen_x_size / 2, screen_y_size / 5 + 96, BLACK,
		"Have fun!");
	
	copy_page (buffer, screen);

	clear_keybuf();
	for (index = 1; index <= 1000; index++)  {
		if ((mouse_b & 1) || keypressed()) break;
		rest(1);
	}
	clear_keybuf();
}
